# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The interface of a top-level application window. """


from traits.api import HasTraits, Instance, List


from pyface.action.api import MenuBarManager, StatusBarManager, ToolBarManager
from pyface.i_image_resource import IImageResource
from pyface.i_window import IWindow


class IApplicationWindow(IWindow):
    """ The interface for a top-level application window.

    The application window has support for a menu bar, tool bar and a status
    bar (all of which are optional).

    Usage
    -----

    Create a sub-class of this class and override the
    :py:meth:`._create_contents` method.
    """

    # 'IApplicationWindow' interface ---------------------------------------

    #: The window icon.  The default is toolkit specific.
    icon = Instance(IImageResource)

    #: The menu bar manager (None iff there is no menu bar).
    menu_bar_manager = Instance(MenuBarManager)

    #: The status bar manager (None iff there is no status bar).
    status_bar_manager = Instance(StatusBarManager)

    #: The tool bar manager (None iff there is no tool bar).
    tool_bar_manager = Instance(ToolBarManager)

    #: If the underlying toolkit supports multiple toolbars, you can use this
    #: list instead of the single ToolBarManager instance above.
    tool_bar_managers = List(ToolBarManager)

    # ------------------------------------------------------------------------
    # Protected 'IApplicationWindow' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        """ Create and return the window's contents.

        Parameters
        ----------
        parent : toolkit control
            The window's toolkit control to be used as the parent for
            widgets in the contents.

        Returns
        -------
        control : toolkit control
            A control to be used for contents of the window.
        """

    def _create_menu_bar(self, parent):
        """ Creates the menu bar (if required).

        Parameters
        ----------
        parent : toolkit control
            The window's toolkit control.
        """

    def _create_status_bar(self, parent):
        """ Creates the status bar (if required).

        Parameters
        ----------
        parent : toolkit control
            The window's toolkit control.
        """

    def _create_tool_bar(self, parent):
        """ Creates the tool bar (if required).

        Parameters
        ----------
        parent : toolkit control
            The window's toolkit control.
        """

    def _create_trim_widgets(self, parent):
        """ Creates the 'trim' widgets (the widgets around the window).

        Parameters
        ----------
        parent : toolkit control
            The window's toolkit control.
        """

    def _set_window_icon(self):
        """ Sets the window icon (if required). """


class MApplicationWindow(HasTraits):
    """ The mixin class that contains common code for toolkit specific
    implementations of the :py:class:`IApplicationWindow` interface.

    Implements: destroy(), _create_trim_widgets()
    """

    # ------------------------------------------------------------------------
    # 'IWidget' interface.
    # ------------------------------------------------------------------------

    def destroy(self):
        """ Destroy the control if it exists. """

        if self.menu_bar_manager is not None:
            self.menu_bar_manager.destroy()

        if self.tool_bar_manager is not None:
            self.tool_bar_manager.destroy()
        for tool_bar_manager in self.tool_bar_managers:
            tool_bar_manager.destroy()

        super(MApplicationWindow, self).destroy()

    # ------------------------------------------------------------------------
    # Protected 'IApplicationWindow' interface.
    # ------------------------------------------------------------------------

    def _create_trim_widgets(self, parent):
        """ Creates the 'trim' widgets (the widgets around the window).

        Parameters
        ----------
        parent : toolkit control
            The window's toolkit control.
        """

        # All of these are optional.
        self._set_window_icon()
        self._create_menu_bar(parent)
        self._create_tool_bar(parent)
        self._create_status_bar(parent)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
from traits.api import Instance

from pyface.action.api import GUIApplicationAction
from pyface.tasks.tasks_application import TasksApplication
from pyface.tasks.task_window_layout import TaskWindowLayout


class TasksApplicationAction(GUIApplicationAction):

    #: The Tasks application the action applies to.
    application = Instance(TasksApplication)


class CreateTaskWindowAction(TasksApplicationAction):
    """ A standard 'New Window' menu action. """

    name = "New Window"
    accelerator = "Ctrl+N"

    #: The task window wayout to use when creating the new window.
    layout = Instance("pyface.tasks.task_window_layout.TaskWindowLayout")

    def perform(self, event=None):
        window = self.application.create_window(layout=self.layout)
        self.application.add_window(window)

    def _layout_default(self):
        if self.application.default_layout:
            layout = self.application.default_layout[0]
        else:
            layout = TaskWindowLayout()
            if self.task_factories:
                layout.items = [self.task_factories[0].id]
        return layout



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" Common toolkit loading utilities and classes

This module provides common code for ETS packages that need to do GUI toolkit
discovery and loading.  The common patterns that ETS has settled on are that
where different GUI toolkits require alternative implementations of features
the toolkit is expected to provide a callable object which takes a relative
module path and an object name, separated by a colon and return the toolkit's
implementation of that object (usually this is a class, but it could be
anything).  The assumption is that this is implemented by objects in
sub-modules of the toolkit, but plugin authors are free to use whatever methods
they like.

Which toolkit to use is specified via the :py:mod:`traits.etsconfig.etsconfig`
package, but if this is not explicitly set by an application at startup or via
environment variables, there needs to be a way of discovering and loading any
available working toolkit implementations.  The default mechanism is via the
now-standard :py:mod:`importlib_metadata` and :py:mod:`setuptools`
"entry point" system.

This module provides three things:

- a function :py:func:`import_toolkit` that attempts to find and load a toolkit
  entry point for a specified toolkit name

- a function :py:func:`find_toolkit` that attempts to find a toolkit entry
  point that works

- a class :py:class:`Toolkit` class that implements the standard logic for
  finding toolkit objects.

These are done in a library-agnostic way so that the same tools can be used
not just for different pyface backends, but also for TraitsUI and ETS
libraries where we need to switch between different GUI toolkit
implementations.

Note that there is no requirement for new toolkit implementations to use this
:py:class:`Toolkit` implementation, but they should be compatible with it.

Default toolkit loading logic
-----------------------------

The :py:func:`find_toolkit` function uses the following logic when attempting
to load toolkits:

- if ETSConfig.toolkit is set, try to load a plugin with a matching name.
  If it succeeds, we are good, and if it fails then we error out.

- after that, we try every 'pyface.toolkit' plugin we can find.  If one
  succeeds, we consider ourselves good, and set the ETSConfig.toolkit
  appropriately.  The order is configurable, and by default will try to load
  the `qt4` toolkit first, `wx` next, then all others in arbitrary order,
  and `null` last.

- finally, if all else fails, we try to load the null toolkit.
"""

import logging
import os
import sys

try:
    # Starting Python 3.8, importlib.metadata is available in the Python
    # standard library.
    import importlib.metadata as importlib_metadata
except ImportError:
    import importlib_metadata

from traits.api import HasTraits, List, ReadOnly, Str, TraitError
from traits.etsconfig.api import ETSConfig


try:
    provisional_toolkit = ETSConfig.provisional_toolkit
except AttributeError:
    from contextlib import contextmanager

    # for backward compatibility
    @contextmanager
    def provisional_toolkit(toolkit_name):
        """ Perform an operation with toolkit provisionally set

        This sets the toolkit attribute of the ETSConfig object set to the
        provided value. If the operation fails with an exception, the toolkit
        is reset to nothing.
        """
        if ETSConfig.toolkit:
            raise AttributeError("ETSConfig toolkit is already set")
        ETSConfig.toolkit = toolkit_name
        try:
            yield
        except:
            # reset the toolkit state
            ETSConfig._toolkit = ""
            raise


logger = logging.getLogger(__name__)


TOOLKIT_PRIORITIES = {"qt4": -2, "wx": -1, "null": float("inf")}
default_priorities = lambda plugin: TOOLKIT_PRIORITIES.get(plugin.name, 0)


class Toolkit(HasTraits):
    """ A basic toolkit implementation for use by specific toolkits.

    This implementation uses pathname mangling to find modules and objects in
    those modules.  If an object can't be found, the toolkit will return a
    class that raises NotImplementedError when it is instantiated.
    """

    #: The name of the package (eg. pyface)
    package = ReadOnly

    #: The name of the toolkit
    toolkit = ReadOnly

    #: The packages to look in for implementations.
    packages = List(Str)

    def __init__(self, package, toolkit, *packages, **traits):
        super(Toolkit, self).__init__(
            package=package, toolkit=toolkit, packages=list(packages), **traits
        )

    def __call__(self, name):
        """ Return the toolkit specific object with the given name.

        Parameters
        ----------
        name : str
            The name consists of the relative module path and the object name
            separated by a colon.
        """
        from importlib import import_module

        mname, oname = name.split(":")
        if not mname.startswith("."):
            mname = "." + mname

        for package in self.packages:
            try:
                module = import_module(mname, package)
            except ImportError as exc:
                # is the error while trying to import package mname or not?
                if all(
                    part not in exc.args[0]
                    for part in mname.split(".")
                    if part
                ):
                    # something else went wrong - let the exception be raised
                    raise

                # Ignore *ANY* errors unless a debug ENV variable is set.
                if "ETS_DEBUG" in os.environ:
                    # Attempt to only skip errors in importing the backend modules.
                    # The idea here is that this only happens when the last entry in
                    # the traceback's stack frame mentions the toolkit in question.
                    import traceback

                    frames = traceback.extract_tb(sys.exc_traceback)
                    filename, lineno, function, text = frames[-1]
                    if not package in filename:
                        raise
            else:
                obj = getattr(module, oname, None)
                if obj is not None:
                    return obj

        toolkit = self.toolkit

        class Unimplemented(object):
            """ An unimplemented toolkit object

            This is returned if an object isn't implemented by the selected
            toolkit.  It raises an exception if it is ever instantiated.
            """

            def __init__(self, *args, **kwargs):
                msg = "the %s %s backend doesn't implement %s"
                raise NotImplementedError(msg % (toolkit, package, name))

        return Unimplemented


def import_toolkit(toolkit_name, entry_point="pyface.toolkits"):
    """ Attempt to import an toolkit specified by an entry point.

    Parameters
    ----------
    toolkit_name : str
        The name of the toolkit we would like to load.
    entry_point : str
        The name of the entry point that holds our toolkits.

    Returns
    -------
    toolkit_object : callable
        A callable object that implements the Toolkit interface.

    Raises
    ------
    RuntimeError
        If no toolkit is found, or if the toolkit cannot be loaded for some
        reason.
    """
    entry_point_group = importlib_metadata.entry_points()[entry_point]
    plugins = [
        plugin for plugin in entry_point_group if plugin.name == toolkit_name
    ]
    if len(plugins) == 0:
        msg = "No {} plugin found for toolkit {}"
        msg = msg.format(entry_point, toolkit_name)
        logger.debug(msg)
        raise RuntimeError(msg)
    elif len(plugins) > 1:
        msg = "multiple %r plugins found for toolkit %r: %s"
        module_names = []
        for plugin in plugins:
            module_names.append(plugin.value.split(":")[0])
        module_names = ", ".join(module_names)
        logger.warning(msg, entry_point, toolkit_name, module_names)

    for plugin in plugins:
        try:
            toolkit_object = plugin.load()
            return toolkit_object
        except (ImportError, AttributeError) as exc:
            msg = "Could not load plugin %r from %r"
            module_name = plugin.value.split(":")[0]
            logger.info(msg, plugin.name, module_name)
            logger.debug(exc, exc_info=True)

    msg = "No {} plugin could be loaded for {}"
    msg = msg.format(entry_point, toolkit_name)
    logger.info(msg)
    raise RuntimeError(msg)


def find_toolkit(entry_point, toolkits=None, priorities=default_priorities):
    """ Find a toolkit that works.

    If ETSConfig is set, then attempt to find a matching toolkit.  Otherwise
    try every plugin for the entry_point until one works.  The ordering of the
    plugins is supplied via the priorities function which should be suitable
    for use as a sorting key function.  If all else fails, explicitly try to
    load the "null" toolkit backend.  If that fails, give up.

    Parameters
    ----------
    entry_point : str
        The name of the entry point that holds our toolkits.
    toolkits : collection of strings
        Only consider toolkits which match the given strings, ignore other
        ones.
    priorities : callable
        A callable function that returns an priority for each plugin.

    Returns
    -------
    toolkit : Toolkit instance
        A callable object that implements the Toolkit interface.

    Raises
    ------
    TraitError
        If no working toolkit is found.
    RuntimeError
        If no ETSConfig.toolkit is set but the toolkit cannot be loaded for
        some reason.
    """
    if ETSConfig.toolkit:
        return import_toolkit(ETSConfig.toolkit, entry_point)

    entry_points = [
        plugin for plugin in importlib_metadata.entry_points()[entry_point]
        if toolkits is None or plugin.name in toolkits
    ]
    for plugin in sorted(entry_points, key=priorities):
        try:
            with ETSConfig.provisional_toolkit(plugin.name):
                toolkit = plugin.load()
                return toolkit
        except (ImportError, AttributeError, RuntimeError) as exc:
            msg = "Could not load %s plugin %r from %r"
            module_name = plugin.value.split(":")[0]
            logger.info(msg, entry_point, plugin.name, module_name)
            logger.debug(exc, exc_info=True)

    # if all else fails, try to import the null toolkit.
    with ETSConfig.provisional_toolkit("null"):
        return import_toolkit("null", entry_point)

    raise TraitError("Could not import any {} toolkit.".format(entry_point))



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import unittest

from traits.testing.api import UnittestTools

from pyface.undo.api import CommandStack, UndoManager
from pyface.undo.tests.testing_commands import SimpleCommand, UnnamedCommand


class TestUndoManager(unittest.TestCase, UnittestTools):

    def setUp(self):
        self.stack_a = CommandStack()
        self.stack_b = CommandStack()
        self.undo_manager = UndoManager()
        self.stack_a.undo_manager = self.undo_manager
        self.stack_b.undo_manager = self.undo_manager

        self.undo_manager.active_stack = self.stack_a

        self.command = SimpleCommand()

    # Command pushing tests ---------------------------------------------------

    def test_undo(self):
        self.assertEqual(self.stack_a._index, -1)
        self.stack_a.push(self.command)
        self.assertEqual(self.stack_a._index, 0)
        with self.assertTraitChanges(
                self.undo_manager, 'stack_updated', count=1):
            self.undo_manager.undo()
        self.assertEqual(self.stack_a._index, -1)

    def test_redo(self):
        self.assertEqual(self.stack_a._index, -1)
        self.stack_a.push(self.command)
        self.undo_manager.undo()
        self.assertEqual(self.stack_a._index, -1)
        with self.assertTraitChanges(
                self.undo_manager, 'stack_updated', count=1):
            self.undo_manager.redo()
        self.assertEqual(self.stack_a._index, 0)

    def test_change_active_stack(self):
        for _ in range(5):
            self.stack_a.push(self.command)
        self.assertEqual(self.stack_a._index, 4)
        self.undo_manager.active_stack = self.stack_b
        for _ in range(5):
            self.stack_b.push(self.command)
        self.assertEqual(self.stack_b._index, 4)
        for _ in range(3):
            self.undo_manager.undo()
        self.undo_manager.redo()

        self.assertEqual(self.stack_a._index, 4)
        self.assertEqual(self.stack_b._index, 2)

    def test_active_stack_clean(self):
        self.assertTrue(self.undo_manager.active_stack_clean)
        self.stack_a.push(self.command)
        self.assertFalse(self.undo_manager.active_stack_clean)
        self.undo_manager.active_stack = None
        self.assertTrue(self.undo_manager.active_stack_clean)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Instance


from pyface.tasks.dock_pane import DockPane


class EnamlDockPane(DockPane):
    """ Create a Dock pane for Enaml Components. """

    # ------------------------------------------------------------------------
    # 'EnamlDockPane' interface
    # ------------------------------------------------------------------------

    #: The Enaml component defining the contents of the DockPane.
    component = Instance("enaml.widgets.toolkit_object.ToolkitObject")

    def create_component(self):
        """ Return an Enaml component defining the contents of the DockPane.

        Returns
        -------
        component : ToolkitObject
        """
        raise NotImplementedError()

    # ------------------------------------------------------------------------
    # 'IDockPane' interface.
    # ------------------------------------------------------------------------

    def create_contents(self, parent):
        """ Return the toolkit-specific control that represents the pane. """

        self.component = self.create_component()

        # Initialize the proxy.
        self.component.initialize()

        # Activate the proxy.
        if not self.component.proxy_is_active:
            self.component.activate_proxy()

        # Fish the Qt control out of the proxy. That's our DockPane content.
        contents = self.component.proxy.widget

        return contents

    # ------------------------------------------------------------------------
    # 'ITaskPane' interface.
    # ------------------------------------------------------------------------

    def destroy(self):
        """ Destroy the toolkit-specific control that represents the pane. """

        control = self.control
        if control is not None:
            control.hide()
            self.component.destroy()
            control.setParent(None)
            control.deleteLater()

        self.control = None
        self.component = None



# (C) Copyright 2007 Riverbank Computing Limited
# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import platform
import sys


from pyface.qt import QtCore, QtGui


from traits.api import Any, Callable, Instance, List, provides, Str, Tuple


from pyface.i_about_dialog import IAboutDialog, MAboutDialog
from pyface.image_resource import ImageResource
from .dialog import Dialog

# The HTML displayed in the QLabel.
_DIALOG_TEXT = """
<html>
  <body>
    <center>
      <table width="100%%" cellspacing="4" cellpadding="0" border="0">
        <tr>
          <td align="center">
          <p>
            <img src="%s" alt="">
          </td>
        </tr>
      </table>

      <p>
      %s<br>
      <br>
      Python %s<br>
      Qt %s<br>
      </p>
      <p>
      %s
      </p>
      <p>
      Copyright &copy; 2003-2021 Enthought, Inc.<br>
      Copyright &copy; 2007 Riverbank Computing Limited
      </p>
  </center>
  </body>
</html>
"""


@provides(IAboutDialog)
class AboutDialog(MAboutDialog, Dialog):
    """ The toolkit specific implementation of an AboutDialog.  See the
    IAboutDialog interface for the API documentation.
    """

    # 'IAboutDialog' interface ---------------------------------------------

    additions = List(Str)

    copyrights = List(Str)

    image = Instance(ImageResource, ImageResource("about"))

    # Private interface ---------------------------------------------------#

    #: A list of connected Qt signals to be removed before destruction.
    #: First item in the tuple is the Qt signal. The second item is the event
    #: handler.
    _connections_to_remove = List(Tuple(Any, Callable))

    # -------------------------------------------------------------------------
    # 'IWidget' interface.
    # -------------------------------------------------------------------------

    def destroy(self):
        while self._connections_to_remove:
            signal, handler = self._connections_to_remove.pop()
            signal.disconnect(handler)

        super(AboutDialog, self).destroy()

    # ------------------------------------------------------------------------
    # Protected 'IDialog' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        label = QtGui.QLabel()

        if self.title == "":
            if parent.parent() is not None:
                title = parent.parent().windowTitle()
            else:
                title = ""

            # Set the title.
            self.title = "About %s" % title

        # Set the page contents.
        label.setText(self._create_html())

        # Create the button.
        buttons = QtGui.QDialogButtonBox()

        if self.ok_label:
            buttons.addButton(self.ok_label, QtGui.QDialogButtonBox.AcceptRole)
        else:
            buttons.addButton(QtGui.QDialogButtonBox.Ok)

        buttons.accepted.connect(parent.accept)
        self._connections_to_remove.append((buttons.accepted, parent.accept))

        lay = QtGui.QVBoxLayout()
        lay.addWidget(label)
        lay.addWidget(buttons)

        parent.setLayout(lay)

    def _create_html(self):
        # Load the image to be displayed in the about box.
        image = self.image.create_image()
        path = self.image.absolute_path

        # The additional strings.
        additions = "<br />".join(self.additions)

        # Get the version numbers.
        py_version = platform.python_version()
        qt_version = QtCore.__version__

        # The additional copyright strings.
        copyrights = "<br />".join(
            ["Copyright &copy; %s" % line for line in self.copyrights]
        )

        return _DIALOG_TEXT % (
            path,
            additions,
            py_version,
            qt_version,
            copyrights,
        )



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import logging
import warnings

import wx
from wx.dataview import (
    DataViewCtrl, DataViewEvent, DataViewItemArray,
    DataViewModel as wxDataViewModel,
    DATAVIEW_CELL_EDITABLE, DATAVIEW_CELL_ACTIVATABLE,
    DV_MULTIPLE, DV_NO_HEADER, EVT_DATAVIEW_SELECTION_CHANGED,
    wxEVT_DATAVIEW_SELECTION_CHANGED
)

from traits.api import Constant, Enum, Instance, observe, provides

from pyface.data_view.i_data_view_widget import (
    IDataViewWidget, MDataViewWidget
)
from pyface.data_view.data_view_errors import DataViewGetError
from pyface.ui.wx.widget import Widget
from .data_view_model import DataViewModel


logger = logging.getLogger(__name__)


# XXX This file is scaffolding and may need to be rewritten

@provides(IDataViewWidget)
class DataViewWidget(MDataViewWidget, Widget):
    """ The Wx implementation of the DataViewWidget. """

    #: What can be selected.
    selection_type = Enum("row")

    #: How selections are modified.
    selection_mode = Enum("extended", "single")

    # Private traits --------------------------------------------------------

    #: The QAbstractItemModel instance used by the view.  This will
    #: usually be a DataViewModel subclass.
    _item_model = Instance(wxDataViewModel)

    # ------------------------------------------------------------------------
    # IDataViewWidget Interface
    # ------------------------------------------------------------------------

    def _create_item_model(self):
        """ Create the DataViewItemModel which wraps the data model. """
        self._item_model = DataViewModel(self.data_model)

    def _get_control_header_visible(self):
        """ Method to get the control's header visibility. """
        return not self.control.GetWindowStyleFlag() & DV_NO_HEADER

    def _set_control_header_visible(self, header_visible):
        """ Method to set the control's header visibility. """
        old_visible = self._get_control_header_visible()
        if header_visible != old_visible:
            self.control.ToggleWindowStyle(DV_NO_HEADER)

    def _get_control_selection_type(self):
        """ Toolkit specific method to get the selection type. """
        return "row"

    def _set_control_selection_type(self, selection_type):
        """ Toolkit specific method to change the selection type. """
        if selection_type != "row":
            warnings.warn(
                "{!r} selection_type not supported in Wx".format(
                    selection_type
                ),
                RuntimeWarning,
            )

    def _get_control_selection_mode(self):
        """ Toolkit specific method to get the selection mode. """
        if self.control.GetWindowStyleFlag() & DV_MULTIPLE:
            return "extended"
        else:
            return "single"

    def _set_control_selection_mode(self, selection_mode):
        """ Toolkit specific method to change the selection mode. """
        if selection_mode not in {'extended', 'single'}:
            warnings.warn(
                "{!r} selection_mode not supported in Wx".format(
                    selection_mode
                ),
                RuntimeWarning,
            )
            return
        old_mode = self._get_control_selection_mode()
        if selection_mode != old_mode:
            self.control.ToggleWindowStyle(DV_MULTIPLE)

    def _get_control_selection(self):
        """ Toolkit specific method to get the selection. """
        return [
            (self._item_model._to_row_index(item), ())
            for item in self.control.GetSelections()
        ]

    def _set_control_selection(self, selection):
        """ Toolkit specific method to change the selection. """
        wx_selection = DataViewItemArray()
        for row, column in selection:
            item = self._item_model._to_item(row)
            wx_selection.append(item)
        self.control.SetSelections(wx_selection)
        if wx.VERSION >= (4, 1):
            if len(wx_selection) > 0:
                item = wx_selection[-1]
            else:
                # a dummy item because  we have nothing specific
                item = self._item_model._to_item((0,))
            event = DataViewEvent(
                wxEVT_DATAVIEW_SELECTION_CHANGED,
                self.control,
                item,
            )
        else:
            event = DataViewEvent(
                wxEVT_DATAVIEW_SELECTION_CHANGED,
            )
        wx.PostEvent(self.control, event)

    def _observe_control_selection(self, remove=False):
        """ Toolkit specific method to watch for changes in the selection. """
        if remove:
            self.control.Unbind(
                EVT_DATAVIEW_SELECTION_CHANGED,
                handler=self._update_selection,
            )
        else:
            self.control.Bind(
                EVT_DATAVIEW_SELECTION_CHANGED,
                self._update_selection,
            )

    # ------------------------------------------------------------------------
    # Widget Interface
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the DataViewWidget's toolkit control. """
        self._create_item_model()

        control = DataViewCtrl(parent)
        control.AssociateModel(self._item_model)
        # required for wxPython refcounting system
        self._item_model.DecRef()

        # create columns for view
        value_type = self._item_model.model.get_value_type([], [])
        try:
            text = value_type.get_text(self._item_model.model, [], [])
        except DataViewGetError:
            text = ''
        except Exception:
            # unexpected error, log and raise
            logger.exception("get header data failed: column ()")
            raise
        control.AppendTextColumn(text, 0, mode=DATAVIEW_CELL_ACTIVATABLE)

        for column in range(self._item_model.GetColumnCount()-1):
            value_type = self._item_model.model.get_value_type([], [column])
            try:
                text = value_type.get_text(self._item_model.model, [], [column])
            except DataViewGetError:
                text = ''
            except Exception:
                # unexpected error, log and raise
                logger.exception(
                    "get header data failed: column (%r,)",
                    column,
                )
                raise

            control.AppendTextColumn(
                text,
                column+1,
                mode=DATAVIEW_CELL_EDITABLE,
            )
        return control

    def destroy(self):
        """ Perform any actions required to destroy the control. """
        super().destroy()
        # ensure that we release the reference to the item model
        self._item_model = None

    # ------------------------------------------------------------------------
    # Private methods
    # ------------------------------------------------------------------------

    # Trait observers

    @observe('data_model', dispatch='ui')
    def update_item_model(self, event):
        if self._item_model is not None:
            self._item_model.model = event.new



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" An action that shows a dialog to allow the user to choose a view. """


from .view_chooser import ViewChooser
from .workbench_action import WorkbenchAction


class ShowViewAction(WorkbenchAction):
    """ An action that shows a dialog to allow the user to choose a view. """

    # 'Action' interface ---------------------------------------------------

    # The action's unique identifier (may be None).
    id = "pyface.workbench.action.show_view"

    # The action's name (displayed on menus/tool bar tools etc).
    name = "Show View"

    # ------------------------------------------------------------------------
    # 'Action' interface.
    # ------------------------------------------------------------------------

    def perform(self, event):
        """ Perform the action. """

        chooser = ViewChooser(window=self.window)

        ui = chooser.edit_traits(parent=self.window.control, kind="livemodal")

        # If the user closes the dialog by using the window manager's close button
        # (e.g. the little [x] in the top corner), ui.result is True, but chooser.view
        # might be None, so we need an explicit check for that.
        if ui.result and chooser.view is not None:
            # This shows the view...
            chooser.view.show()

            # ... and this makes it active (brings it to the front, gives it
            # focus etc).
            chooser.view.activate()

        return



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


from .edit_image_renderer import EditImageRenderer
from .grid_cell_renderer import GridCellRenderer


class EditRenderer(GridCellRenderer):
    def __init__(self, **traits):

        # base-class constructor
        super(EditRenderer, self).__init__(**traits)

        # initialize the renderer, if it hasn't already been initialized
        if self.renderer is None:
            self.renderer = EditImageRenderer()

    def on_left_click(self, grid, row, col):
        """ Calls edit_traits on the object represented by the row. """

        obj = grid.model.get_rows_drag_value([row])[0]

        # allow editting if the obj does not have an editable trait
        # or if the editable trait is True

        if (not hasattr(obj, "editable")) or obj.editable:
            obj.edit_traits(kind="live")



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


""" The handler used to restore editors.
"""


import logging


from pyface.dock.api import SetStructureHandler


logger = logging.getLogger(__name__)


class EditorSetStructureHandler(SetStructureHandler):
    """ The handler used to restore editors.

    This is part of the 'dock window' API. It is used to resolve dock control
    Ids when setting the structure of a dock window.

    """

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, window_layout, editor_mementos):
        """ Creates a new handler. """

        self.window_layout = window_layout
        self.editor_mementos = editor_mementos

        return

    # ------------------------------------------------------------------------
    # 'SetStructureHandler' interface.
    # ------------------------------------------------------------------------

    def resolve_id(self, id):
        """ Resolves an unresolved dock control id. """

        window_layout = self.window_layout
        window = window_layout.window

        try:
            # Get the memento for the editor with this Id.
            memento = self._get_editor_memento(id)

            # Ask the editor manager to create an editor from the memento.
            editor = window.editor_manager.set_editor_memento(memento)

            # Get the editor's toolkit-specific control.
            #
            # fixme: This is using a 'private' method on the window layout.
            # This may be ok since this structure handler is really part of the
            # layout!
            control = window_layout._wx_get_editor_control(editor)

            # fixme: This is ugly manipulating the editors list from in here!
            window.editors.append(editor)

        except:
            logger.warning("could not restore editor [%s]", id)
            control = None

        return control

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _get_editor_memento(self, id):
        """ Return the editor memento for the editor with the specified Id.

        Raises a 'ValueError' if no such memento exists.

        """

        editor_memento = self.editor_mementos.get(id)
        if editor_memento is None:
            raise ValueError("no editor memento with Id %s" % id)

        return editor_memento



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Test for pyface.api """

import unittest

from traits.etsconfig.api import ETSConfig

is_wx = (ETSConfig.toolkit == 'wx')
is_qt = ETSConfig.toolkit.startswith('qt')


class TestApi(unittest.TestCase):
    """ Test importable items in any environment."""

    def test_api_importable(self):
        # make sure api is importable with the most minimal
        # required dependencies, including in the absence of toolkit backends.
        from pyface import api   # noqa: F401


@unittest.skipIf(not is_qt, "This test is for qt.")
class TestApiQt(unittest.TestCase):
    """ Test importable items in a Qt environment."""

    def test_importable_items_minimal(self):
        # Test items should be importable in a minimal Qt environment
        # Pygments is excused. Attempt to import PythonEditor or PythonShell
        # will fail in an environment without pygments will fail, just as it
        # would if these items were imported directly from the corresponding
        # subpackages.
        from pyface.api import (   # noqa: F401
            AboutDialog,
            Alignment,
            Application,
            ApplicationWindow,
            Border,
            CANCEL,
            Clipboard,
            ConfirmationDialog,
            Dialog,
            DirectoryDialog,
            ExpandablePanel,
            FileDialog,
            Filter,
            GUI,
            GUIApplication,
            HasBorder,
            HasMargin,
            HeadingText,
            Image,
            ImageCache,
            ImageResource,
            ImageWidget,
            KeyPressedEvent,
            LayeredPanel,
            MDIApplicationWindow,
            MDIWindowMenu,
            Margin,
            MessageDialog,
            MultiToolbarWindow,
            NO,
            OK,
            ProgressDialog,
            # PythonEditor and PythonShell are omitted
            SingleChoiceDialog,
            Sorter,
            SplashScreen,
            SplitApplicationWindow,
            SplitDialog,
            SplitPanel,
            SystemMetrics,
            Widget,
            Window,
            YES,
            beep,
            choose_one,
            clipboard,
            confirm,
            error,
            information,
            warning,
        )

    def test_python_editor_python_shell_importable(self):
        # If pygments is in the environment, PythonEditor and PythonShell
        # should be importable.
        try:
            import pygments        # noqa: F401
        except ImportError:
            raise self.skipTest("This test requires pygments.")

        from pyface.api import (   # noqa: F401
            PythonEditor,
            PythonShell,
        )


@unittest.skipIf(not is_wx, "This test is for wx.")
class TestApiWx(unittest.TestCase):
    """ Test importable items in a wx environment."""

    def test_importable_items(self):
        # These items should always be importable for wx environment
        from pyface.api import (   # noqa: F401
            AboutDialog,
            Alignment,
            Application,
            ApplicationWindow,
            Border,
            CANCEL,
            Clipboard,
            ConfirmationDialog,
            Dialog,
            DirectoryDialog,
            ExpandablePanel,
            FileDialog,
            Filter,
            GUI,
            GUIApplication,
            HasBorder,
            HasMargin,
            HeadingText,
            Image,
            ImageCache,
            ImageResource,
            ImageWidget,
            KeyPressedEvent,
            LayeredPanel,
            MDIApplicationWindow,
            MDIWindowMenu,
            Margin,
            MessageDialog,
            MultiToolbarWindow,
            NO,
            OK,
            ProgressDialog,
            PythonEditor,
            PythonShell,
            SingleChoiceDialog,
            Sorter,
            SplashScreen,
            SplitApplicationWindow,
            SplitDialog,
            SplitPanel,
            SystemMetrics,
            Widget,
            Window,
            YES,
            beep,
            choose_one,
            clipboard,
            confirm,
            error,
            fix_introspect_bug,
            information,
            warning,
        )



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Int
from pyface.undo.api import AbstractCommand


class SimpleCommand(AbstractCommand):
    """ Simplest command possible operating on an integer. """

    name = "Increment by 1"

    data = Int()

    def do(self):
        self.redo()

    def redo(self):
        self.data += 1

    def undo(self):
        self.data -= 1


class UnnamedCommand(SimpleCommand):
    name = ""


class MergeableCommand(SimpleCommand):

    name = "Increment"

    amount = Int(1)

    def do(self):
        self.redo()

    def redo(self):
        self.data += self.amount

    def undo(self):
        self.data -= self.amount

    def merge(self, other):
        if not isinstance(other, MergeableCommand):
            return False
        self.data += other.amount
        self.amount += other.amount
        return True



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A header for an entry in a collection of expandables. The header
provides a visual indicator of the current state, a text label, and a
'remove' button. """


import wx


from traits.api import Instance, Event, Str, Bool


from pyface.wx.util.font_helper import new_font_like
from .image_resource import ImageResource
from .widget import Widget


class ExpandableHeader(Widget):
    """ A header for an entry in a collection of expandables. The header
    provides a visual indicator of the current state, a text label, and a
    'remove' button. """

    # The title of the panel.
    title = Str("Panel")

    # The carat image to show when the panel is collapsed.
    collapsed_carat_image = Instance(
        ImageResource, ImageResource("carat_closed")
    )
    # The carat image to show when the panel is expanded.
    expanded_carat_image = Instance(ImageResource, ImageResource("carat_open"))
    # The backing header image when the mouse is elsewhere
    header_bar_image = Instance(ImageResource, ImageResource("panel_gradient"))
    # The backing header image when the mouse is over
    header_mouseover_image = Instance(
        ImageResource, ImageResource("panel_gradient_over")
    )

    # The carat image to show when the panel is expanded.
    remove_image = Instance(ImageResource, ImageResource("close"))

    # Represents the current state of the button. True means pressed.
    state = Bool(False)

    # Events ----

    # The panel has been expanded or collapsed
    panel_expanded = Event()

    _CARAT_X = 4
    _CARAT_Y = 2
    _TEXT_Y = 0
    _TEXT_X_OFFSET = 10

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, parent, container, **traits):
        """ Creates the panel. """

        # Base class constructor.
        super(ExpandableHeader, self).__init__(**traits)

        # Create the toolkit-specific control that represents the widget.
        self.control = self._create_control(parent)

        self._container = container
        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """

        collapsed_carat = self.collapsed_carat_image.create_image()
        self._collapsed_bmp = collapsed_carat.ConvertToBitmap()
        self._carat_w = self._collapsed_bmp.GetWidth()

        expanded_carat = self.expanded_carat_image.create_image()
        self._expanded_bmp = expanded_carat.ConvertToBitmap()

        header_bar = self.header_bar_image.create_image()
        self._header_bmp = header_bar.ConvertToBitmap()

        header_bar_over = self.header_mouseover_image.create_image()
        self._header_mouseover_bmp = header_bar_over.ConvertToBitmap()

        self._background_bmp = self._header_bmp

        close_image = self.remove_image.create_image()
        self._remove_bmp = close_image.ConvertToBitmap()

        # create our panel and initialize it appropriately
        sizer = wx.BoxSizer(wx.VERTICAL)
        panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)

        # needed on GTK systems for EVT_ERASE_BACKGROUND to work
        panel.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)

        # create the remove button
        remove = wx.BitmapButton(
            panel, -1, self._remove_bmp, style=0, pos=(-1, 3)
        )
        sizer.Add(remove, 0, wx.ALIGN_RIGHT, 5)

        # Create a suitable font.
        self._font = new_font_like(
            wx.NORMAL_FONT, point_size=wx.NORMAL_FONT.GetPointSize() - 1
        )

        height = self._get_preferred_height(parent, self.title, self._font)
        panel.SetSize((-1, height))

        panel.Bind(wx.EVT_ERASE_BACKGROUND, self._on_erase_background)
        panel.Bind(wx.EVT_ENTER_WINDOW, self._on_enter_leave)
        panel.Bind(wx.EVT_LEAVE_WINDOW, self._on_enter_leave)
        panel.Bind(wx.EVT_LEFT_DOWN, self._on_down)
        panel.Bind(wx.EVT_RIGHT_DOWN, self._on_down)

        panel.Bind(wx.EVT_BUTTON, self._on_remove)

        return panel

    def _get_preferred_height(self, parent, text, font):
        """ Calculates the preferred height of the widget. """

        dc = wx.MemoryDC()

        dc.SetFont(font)
        text_w, text_h = dc.GetTextExtent(text)
        text_h = text_h + self._TEXT_Y

        # add in width of buttons
        carat_h = self._collapsed_bmp.GetHeight() + self._CARAT_Y

        return max(text_h, carat_h)

    def _draw_carat_button(self, dc):
        """ Draws the button at the correct coordinates. """

        if self.state:
            bmp = self._expanded_bmp
        else:
            bmp = self._collapsed_bmp

        dc.DrawBitmap(bmp, self._CARAT_X, self._CARAT_Y, True)

    def _tile_background_image(self, dc, width, height):
        """ Tiles the background image. """

        w = self._background_bmp.GetWidth()
        h = self._background_bmp.GetHeight()

        x = 0
        while x < width:
            y = 0
            while y < height:
                dc.DrawBitmap(self._background_bmp, x, y)

                y = y + h

            x = x + w

    def _draw_title(self, dc):
        """ Draws the text label for the header. """
        dc.SetFont(self._font)

        # Render the text.
        dc.DrawText(
            self.title, self._carat_w + self._TEXT_X_OFFSET, self._TEXT_Y
        )

    def _draw(self, dc):
        """ Draws the control. """

        size = self.control.GetClientSize()

        # Tile the background image.
        self._tile_background_image(dc, size.width, size.height)

        self._draw_title(dc)

        # Draw the carat button
        self._draw_carat_button(dc)

        return

    # ------------------------------------------------------------------------
    # wx event handlers.
    # ------------------------------------------------------------------------

    def _on_erase_background(self, event):
        """ Called when the background of the panel is erased. """

        # print 'ImageButton._on_erase_background'
        dc = event.GetDC()
        self._draw(dc)

    def _on_enter_leave(self, event):
        """ Called when button is pressed. """

        # print 'ExpandableHeader._on_enter_leave'
        if event.Entering():
            self._background_bmp = self._header_mouseover_bmp
        else:
            self._background_bmp = self._header_bmp

        self.control.Refresh()
        event.Skip()

    def _on_down(self, event):
        """ Called when button is pressed. """

        # print 'ImageButton._on_down'
        self.state = not self.state
        self.control.Refresh()

        # fire an event so any listeners can pick up the change
        self.panel_expanded = self
        event.Skip()

    def _on_remove(self, event):
        """ Called when remove button is pressed. """

        self._container.remove_panel(self.title)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from unittest import TestCase, expectedFailure
from unittest.mock import Mock

from pyface.color import Color
from pyface.data_view.abstract_data_model import DataViewSetError
from pyface.data_view.value_types.color_value import ColorValue


class TestColorValue(TestCase):

    def setUp(self):
        self.model = Mock()
        self.model.get_value = Mock(
            return_value=Color(rgba=(0.4, 0.2, 0.6, 0.8)),
        )
        self.model.can_set_value = Mock(return_value=True)
        self.model.set_value = Mock()

    def test_defaults(self):
        value = ColorValue()
        self.assertTrue(value.is_editable)

    def test_is_valid(self):
        value = ColorValue()
        self.assertTrue(value.is_valid(None, [0], [0], Color()))

    def test_get_editor_value(self):
        value = ColorValue()
        editable = value.get_editor_value(self.model, [0], [0])
        self.assertEqual(editable, "#663399CC")

    def test_set_editor_value(self):
        value = ColorValue()
        value.set_editor_value(self.model, [0], [0], "#3399CC66")
        self.model.set_value.assert_called_once_with(
            [0],
            [0],
            Color(rgba=(0.2, 0.6, 0.8, 0.4)),
        )

    def test_get_text(self):
        value = ColorValue()
        editable = value.get_text(self.model, [0], [0])
        self.assertEqual(editable, "#663399CC")

    def test_set_text(self):
        value = ColorValue()
        value.set_text(self.model, [0], [0], "red")
        self.model.set_value.assert_called_once_with(
            [0],
            [0],
            Color(rgba=(1.0, 0.0, 0.0, 1.0)),
        )

    def test_set_text_error(self):
        value = ColorValue()
        with self.assertRaises(DataViewSetError):
            value.set_text(self.model, [0], [0], "not a real color")

    def test_set_text_no_set_value(self):
        self.model.can_set_value = Mock(return_value=False)
        value = ColorValue()
        value.set_text(self.model, [0], [0], "red")
        self.model.set_value.assert_called_once_with(
            [0],
            [0],
            Color(rgba=(1.0, 0.0, 0.0, 1.0)),
        )

    def test_get_color(self):
        value = ColorValue()
        editable = value.get_color(self.model, [0], [0])
        self.assertEqual(editable, Color(rgba=(0.4, 0.2, 0.6, 0.8)))



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


"""

API for the ``pyface.action`` subpackage.

Actions
-------

- :class:`~.Action`
- :class:`~.FieldAction`
- :class:`~.GUIApplicationAction`
- :class:`~.ListeningAction`
- :class:`~.TraitsUIWidgetAction`
- :class:`~.WindowAction`

Action Controller
-----------------

- :class:`~.ActionController`

Action Event
------------

- :class:`~.ActionEvent`

Action Managers
---------------

- :class:`~.ActionManager`
- ``MenuManager``
- ``MenuBarManager``
- ``StatusBarManager``
- ``ToolBarManager``
- ``ToolPaletteManager``, only for the Wx toolkit.

Action Manager Items
--------------------

- :class:`~.ActionManagerItem`
- :class:`~.ActionItem`

Layout support
--------------

- :class:`~.Group`
- :class:`~.Separator`

Useful Application and Window actions
-------------------------------------

- :class:`~.AboutAction`
- :class:`~.CloseActiveWindowAction`
- :class:`~.CreateWindowAction`
- :class:`~.ExitAction`
- :class:`~.CloseWindowAction`

"""

from .action import Action
from .action_controller import ActionController
from .action_event import ActionEvent
from .action_item import ActionItem
from .action_manager import ActionManager
from .action_manager_item import ActionManagerItem
from .field_action import FieldAction
from .group import Group, Separator
from .gui_application_action import (
    AboutAction,
    CloseActiveWindowAction,
    CreateWindowAction,
    ExitAction,
    GUIApplicationAction,
)
from .listening_action import ListeningAction
from .menu_manager import MenuManager
from .menu_bar_manager import MenuBarManager
from .status_bar_manager import StatusBarManager
from .tool_bar_manager import ToolBarManager
from .traitsui_widget_action import TraitsUIWidgetAction
from .window_action import CloseWindowAction, WindowAction

# This part of the module handles widgets that are still wx specific.  This
# will all be removed when everything has been ported to PyQt and pyface
# becomes toolkit agnostic.

from traits.etsconfig.api import ETSConfig

if ETSConfig.toolkit == "wx":
    from .tool_palette_manager import ToolPaletteManager



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought util package component>

from pyface.timer.timer import CallbackTimer, Timer


class DoLaterTimer(Timer):
    """ Performs a callback once at a later time.

    This is not used by the `do_later` functions and is only provided for
    backwards compatibility of the API.
    """

    #: The perform the callback once.
    repeat = 1

    def __init__(self, interval, callable, args, kw_args):
        # Adapt the old DoLaterTimer initializer to the Timer initializer.
        super(DoLaterTimer, self).__init__(
            interval, callable, *args, **kw_args
        )


def do_later(callable, *args, **kwargs):
    """ Does something 50 milliseconds from now.

    Parameters
    ----------
    callable : callable
        The callable to call in 50ms time.
    *args, \**kwargs : tuple, dict
        Arguments to be passed through to the callable.
    """
    return CallbackTimer.single_shot(
        interval=0.05, callback=callable, args=args, kwargs=kwargs
    )


def do_after(interval, callable, *args, **kwargs):
    """ Does something after some specified time interval.

    Parameters
    ----------
    interval : float
        The time interval in milliseconds to wait before calling.
    callable : callable
        The callable to call.
    \*args
        Positional arguments to be passed through to the callable.
    \*\*kwargs
        Keyword arguments to be passed through to the callable.
        Arguments to be passed through to the callable.
    """
    return CallbackTimer.single_shot(
        interval=interval / 1000.0, callback=callable, args=args, kwargs=kwargs
    )



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
# (C) Copyright 2007 Riverbank Computing Limited
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" Initialize this backend.
"""

from pyface.base_toolkit import Toolkit

toolkit_object = Toolkit("pyface", "null", "pyface.ui.null")



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import logging

import os


import wx

# Logger.
logger = logging.getLogger(__name__)

# Multiple AUI versions are no longer supported; the C version in wx.aui is not
# capable of supporting the windowing flexibility needed by tasks. Therefore,
# only AGW's pure-python AUI implementation is used.

from wx.lib.agw import aui

# AGW's library does need some patching for some usability differences desired
# for pyface but not for the standard wxPython version


class PyfaceAuiNotebook(aui.AuiNotebook):
    if wx.version() >= "3.":
        SetPageToolTip = aui.AuiNotebook.SetPageTooltip
        GetPageToolTip = aui.AuiNotebook.GetPageTooltip


class PyfaceAuiManager(aui.AuiManager):
    # The standard AuiManager dock resizing attempts to adjust all the docks to
    # provide some sort of best fit, but when there are more than two panes in
    # a dock it isn't very intuitive. The modifications to these three methods
    # tries to keep as many sizers fixes as it can and only adjust the one that
    # is added.

    def CalculateDockSizerLimits(self, dock):
        # Replacement for default calculation for min/max dock sizes. Instead
        # of adjusting the sizes of all the docks, only adjusts one to make the
        # dock insertion process a little more like what the user expected.

        docks, panes = aui.CopyDocksAndPanes2(self._docks, self._panes)

        sash_size = self._art.GetMetric(aui.AUI_DOCKART_SASH_SIZE)
        caption_size = self._art.GetMetric(aui.AUI_DOCKART_CAPTION_SIZE)
        opposite_size = self.GetOppositeDockTotalSize(
            docks, dock.dock_direction
        )

        for tmpDock in docks:

            if (
                tmpDock.dock_direction == dock.dock_direction
                and tmpDock.dock_layer == dock.dock_layer
                and tmpDock.dock_row == dock.dock_row
            ):

                tmpDock.size = 1
                break
        neighbor_docks = []
        horizontal = (
            dock.dock_direction == aui.AUI_DOCK_LEFT
            or dock.dock_direction == aui.AUI_DOCK_RIGHT
        )
        right_or_down = (
            dock.dock_direction == aui.AUI_DOCK_RIGHT
            or dock.dock_direction == aui.AUI_DOCK_BOTTOM
        )
        for d in docks:
            if (
                d.dock_direction == dock.dock_direction
                and d.dock_layer == dock.dock_layer
            ):
                if horizontal:
                    neighbor_docks.append((d.rect.x, d.rect.width))
                else:
                    neighbor_docks.append((d.rect.y, d.rect.height))
        neighbor_docks.sort()

        sizer, panes, docks, uiparts = self.LayoutAll(
            panes, docks, [], True, False
        )
        client_size = self._frame.GetClientSize()
        sizer.SetDimension(0, 0, client_size.x, client_size.y)
        sizer.Layout()

        for part in uiparts:

            part.rect = wx.Rect(
                part.sizer_item.GetPosition(), part.sizer_item.GetSize()
            )
            if part.type == aui.AuiDockUIPart.typeDock:
                part.dock.rect = part.rect

        sizer.Destroy()
        new_dock = None

        for tmpDock in docks:
            if (
                tmpDock.dock_direction == dock.dock_direction
                and tmpDock.dock_layer == dock.dock_layer
                and tmpDock.dock_row == dock.dock_row
            ):

                new_dock = tmpDock
                break

        partnerDock = self.GetPartnerDock(dock)

        if partnerDock:
            if horizontal:
                pos = dock.rect.x
                size = dock.rect.width
            else:
                pos = dock.rect.y
                size = dock.rect.height

            min_pos = pos
            max_pos = pos + size
            if right_or_down:
                for p, s in neighbor_docks:
                    if p >= pos:
                        max_pos = p + s - sash_size
                        break
                    else:
                        min_pos = p + sash_size
            else:
                for p, s in neighbor_docks:
                    if p > pos:
                        max_pos = p + s - sash_size
                        break
                    else:
                        min_pos = p + sash_size

            return min_pos, max_pos

        direction = new_dock.dock_direction

        if direction == aui.AUI_DOCK_LEFT:
            minPix = new_dock.rect.x + new_dock.rect.width
            maxPix = client_size.x - opposite_size - sash_size

        elif direction == aui.AUI_DOCK_TOP:
            minPix = new_dock.rect.y + new_dock.rect.height
            maxPix = client_size.y - opposite_size - sash_size

        elif direction == aui.AUI_DOCK_RIGHT:
            minPix = opposite_size
            maxPix = new_dock.rect.x - sash_size

        elif direction == aui.AUI_DOCK_BOTTOM:
            minPix = opposite_size
            maxPix = new_dock.rect.y - sash_size

        return minPix, maxPix

    def GetPartnerDockFromPos(self, dock, point):
        """Get the neighboring dock located at the given position, used to
        find the other dock that is going to change size when resizing the
        specified dock.
        """
        horizontal = (
            dock.dock_direction == aui.AUI_DOCK_LEFT
            or dock.dock_direction == aui.AUI_DOCK_RIGHT
        )
        right_or_down = (
            dock.dock_direction == aui.AUI_DOCK_RIGHT
            or dock.dock_direction == aui.AUI_DOCK_BOTTOM
        )
        if horizontal:
            pos = point.x
        else:
            pos = point.y
        neighbor_docks = []
        for d in self._docks:
            if (
                d.dock_direction == dock.dock_direction
                and d.dock_layer == dock.dock_layer
            ):
                if horizontal:
                    neighbor_docks.append((d.rect.x, d.rect.width, d))
                else:
                    neighbor_docks.append((d.rect.y, d.rect.height, d))
        neighbor_docks.sort()
        last = None
        if right_or_down:
            for p, s, d in neighbor_docks:
                if pos < p + s:
                    if d.dock_row == dock.dock_row:
                        d = last
                    break
                last = d
        else:
            neighbor_docks.reverse()
            for p, s, d in neighbor_docks:
                if pos > p:
                    if d.dock_row == dock.dock_row:
                        d = last
                    break
                last = d
        return d

    def RestrictResize(self, clientPt, screenPt, createDC):
        """ Common method between :meth:`DoEndResizeAction` and :meth:`OnLeftUp_Resize`. """

        dock = self._action_part.dock
        pane = self._action_part.pane

        if createDC:
            if wx.Platform == "__WXMAC__":
                dc = wx.ClientDC(self._frame)
            else:
                dc = wx.ScreenDC()

            aui.DrawResizeHint(dc, self._action_rect)
            self._action_rect = wx.Rect()

        newPos = clientPt - self._action_offset

        if self._action_part.type == aui.AuiDockUIPart.typeDockSizer:
            minPix, maxPix = self.CalculateDockSizerLimits(dock)
        else:
            if not self._action_part.pane:
                return
            minPix, maxPix = self.CalculatePaneSizerLimits(dock, pane)

        if self._action_part.orientation == wx.HORIZONTAL:
            newPos.y = aui.Clip(newPos.y, minPix, maxPix)
        else:
            newPos.x = aui.Clip(newPos.x, minPix, maxPix)

        if self._action_part.type == aui.AuiDockUIPart.typeDockSizer:
            partner = self.GetPartnerDockFromPos(dock, newPos)
            sash_size = self._art.GetMetric(aui.AUI_DOCKART_SASH_SIZE)
            button_size = self._art.GetMetric(aui.AUI_DOCKART_PANE_BUTTON_SIZE)
            new_dock_size = 0
            direction = dock.dock_direction

            if direction == aui.AUI_DOCK_LEFT:
                new_dock_size = newPos.x - dock.rect.x

            elif direction == aui.AUI_DOCK_TOP:
                new_dock_size = newPos.y - dock.rect.y

            elif direction == aui.AUI_DOCK_RIGHT:
                new_dock_size = (
                    dock.rect.x + dock.rect.width - newPos.x - sash_size
                )

            elif direction == aui.AUI_DOCK_BOTTOM:
                new_dock_size = (
                    dock.rect.y + dock.rect.height - newPos.y - sash_size
                )

            delta = new_dock_size - dock.size
            if delta < -dock.size + sash_size:
                delta = -dock.size + sash_size
            elif -button_size < delta < button_size:
                delta = button_size * (1 if delta > 0 else -1)

            if partner:
                if delta > partner.size - sash_size:
                    delta = partner.size - sash_size
                partner.size -= delta
            dock.size += delta
            self.Update()

        else:

            # determine the new pixel size that the user wants
            # this will help us recalculate the pane's proportion
            if dock.IsHorizontal():
                oldPixsize = pane.rect.width
                newPixsize = oldPixsize + newPos.x - self._action_part.rect.x

            else:
                oldPixsize = pane.rect.height
                newPixsize = oldPixsize + newPos.y - self._action_part.rect.y

            totalPixsize, totalProportion = self.GetTotalPixSizeAndProportion(
                dock
            )
            partnerPane = self.GetPartnerPane(dock, pane)

            # prevent division by zero
            if totalPixsize <= 0 or totalProportion <= 0 or not partnerPane:
                return

            # adjust for the surplus
            while (
                oldPixsize > 0
                and totalPixsize > 10
                and oldPixsize * totalProportion / totalPixsize
                < pane.dock_proportion
            ):

                totalPixsize -= 1

            # calculate the new proportion of the pane

            newProportion = newPixsize * totalProportion / totalPixsize
            newProportion = aui.Clip(newProportion, 1, totalProportion)
            deltaProp = newProportion - pane.dock_proportion

            if partnerPane.dock_proportion - deltaProp < 1:
                deltaProp = partnerPane.dock_proportion - 1
                newProportion = pane.dock_proportion + deltaProp

            # borrow the space from our neighbor pane to the
            # right or bottom (depending on orientation)
            partnerPane.dock_proportion -= deltaProp
            pane.dock_proportion = newProportion

            self.Update()

        return True

    def UpdateWithoutLayout(self):
        """If the layout in the AUI manager is not changing, this can be called
        to refresh all the panes but preventing a big time usage doing a re-
        layout that isn't necessary.
        """
        pane_count = len(self._panes)

        for ii in range(pane_count):
            p = self._panes[ii]
            if p.window and p.IsShown() and p.IsDocked():
                p.window.Refresh()
                p.window.Update()

        if wx.Platform == "__WXMAC__":
            self._frame.Refresh()
        else:
            self.Repaint()



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest
from unittest import mock

from pyface.ui.qt4.workbench.split_tab_widget import SplitTabWidget
from pyface.ui.qt4.workbench.workbench_window_layout import (
    WorkbenchWindowLayout,
)


class TestWorkbenchWindowLayout(unittest.TestCase):
    def test_change_of_active_qt_editor(self):
        # Test error condition for enthought/mayavi#321

        mock_split_tab_widget = mock.Mock(spec=SplitTabWidget)

        layout = WorkbenchWindowLayout(_qt4_editor_area=mock_split_tab_widget)

        class DummyEvent:
            def __init__(self, new):
                self.new = new

        # This should not throw
        layout._qt4_active_editor_changed(DummyEvent(new=None))
        self.assertEqual(mock_split_tab_widget.setTabTextColor.called, False)

        mock_active_editor = mock.Mock()
        layout._qt4_active_editor_changed(mock_active_editor)

        self.assertEqual(mock_split_tab_widget.setTabTextColor.called, True)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A renderer which will display a cell-specific image in addition to some
    text displayed in the same way the standard string renderer normally
    would. """


import wx

from wx.grid import GridCellRenderer
from wx.grid import GridCellStringRenderer
from wx import SOLID, Brush, Rect, TRANSPARENT_PEN


class GridCellImageRenderer(GridCellRenderer):
    """ A renderer which will display a cell-specific image in addition to some
        text displayed in the same way the standard string renderer normally
        would. """

    def __init__(self, provider=None):
        """ Build a new PyGridCellImageRenderer.

            'provider', if provided, should implement
            get_image_for_cell(row, col) to specify an image to appear
            in the cell at row, col. """

        GridCellRenderer.__init__(self)

        # save the string renderer to use for text.
        self._string_renderer = GridCellStringRenderer()

        self._provider = provider

        return

    # ------------------------------------------------------------------------
    # GridCellRenderer interface.
    # ------------------------------------------------------------------------
    def Draw(self, grid, attr, dc, rect, row, col, isSelected):
        """ Draw the appropriate icon into the specified grid cell. """

        # clear the cell first
        if isSelected:
            bgcolor = grid.GetSelectionBackground()
        else:
            bgcolor = grid.GetCellBackgroundColour(row, col)

        dc.SetBackgroundMode(SOLID)
        dc.SetBrush(Brush(bgcolor, SOLID))
        dc.SetPen(TRANSPARENT_PEN)
        dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height)

        # find the correct image for this cell
        bmp = self._get_image(grid, row, col)
        # find the correct text for this cell
        text = self._get_text(grid, row, col)

        # figure out placement -- we try to center things!
        # fixme: we should be responding to the horizontal/vertical
        #        alignment info in the attr object!
        size = self.GetBestSize(grid, attr, dc, row, col)

        halign, valign = attr.GetAlignment()

        # width first
        wdelta = rect.width - size.GetWidth()
        x = rect.x
        if halign == wx.ALIGN_CENTRE and wdelta > 0:
            x += wdelta / 2

        # now height
        hdelta = rect.height - size.GetHeight()

        y = rect.y
        if valign == wx.ALIGN_CENTRE and hdelta > 0:
            y += hdelta / 2

        dc.SetClippingRegion(*rect)

        if bmp is not None:
            # now draw our image into it
            dc.DrawBitmap(bmp, x, y, 1)
            x += bmp.GetWidth()

        if text is not None and text != "":
            width = rect.x + rect.width - x
            height = rect.y + rect.height - y
            # draw any text that should be included
            new_rect = Rect(x, y, width, height)
            self._string_renderer.Draw(
                grid, attr, dc, new_rect, row, col, isSelected
            )

        dc.DestroyClippingRegion()

    def GetBestSize(self, grid, attr, dc, row, col):
        """ Determine best size for the cell. """

        # find the correct image
        bmp = self._get_image(grid, row, col)

        if bmp is not None:
            bmp_size = wx.Size(bmp.GetWidth(), bmp.GetHeight())
        else:
            bmp_size = wx.Size(0, 0)

        # find the correct text for this cell
        text = self._get_text(grid, row, col)

        if text is not None:
            text_size = self._string_renderer.GetBestSize(
                grid, attr, dc, row, col
            )
        else:
            text_size = wx.Size(0, 0)

        result = wx.Size(
            bmp_size.width + text_size.width,
            max(bmp_size.height, text_size.height),
        )

        return result

    def Clone(self):
        return GridCellImageRenderer(self._provider)

    # ------------------------------------------------------------------------
    # protected 'GridCellIconRenderer' interface.
    # ------------------------------------------------------------------------
    def _get_image(self, grid, row, col):
        """ Returns the correct bmp for the data at row, col. """
        bmp = None
        if self._provider is not None and hasattr(
            self._provider, "get_image_for_cell"
        ):
            # get the image from the specified provider
            img = self._provider.get_image_for_cell(grid, row, col)
            if img is not None:
                bmp = img.create_bitmap()
            else:
                bmp = None

        return bmp

    def _get_text(self, grid, row, col):
        """ Returns the correct text for the data at row, col. """

        text = None
        if self._provider is not None and hasattr(
            self._provider, "get_text_for_cell"
        ):
            # get the image from the specified provider
            text = self._provider.get_text_for_cell(grid, row, col)
        else:
            text = grid.GetCellValue(row, col)

        return text



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Provides an AbstractValueType ABC for Pyface data models.

This module provides an ABC for data view value types, which are responsible
for adapting raw data values as used by the data model's ``get_value`` and
``set_value`` methods to the data channels that the data view expects, such
as text, color, icons, etc.

It is up to the data view to take this standardized data and determine what
and how to actually display it.
"""

from enum import IntEnum

from traits.api import ABCHasStrictTraits, Event, observe

from pyface.color import Color
from .data_view_errors import DataViewSetError


class CheckState(IntEnum):
    "Possible checkbox states"
    # XXX in the future this may need a "partial" state, see Pyface #695
    UNCHECKED = 0
    CHECKED = 1


class AbstractValueType(ABCHasStrictTraits):
    """ A value type converts raw data into data channels.

    The data channels are editor value, text, color, image, and description.
    The data channels are used by other parts of the code to produce the actual
    display.

    Subclasses should mark traits that potentially affect the display of values
    with ``update_value_type=True`` metdadata, or alternatively fire the
    ``updated`` event when the state of the value type changes.

    Each data channel is set up to have a method which returns whether there
    is a value for the channel, a second method which returns the value,
    and an optional third method which sets the channel value.  These methods
    should not raise an Exception, eveen when called inappropriately (eg.
    calling a "get" method after a "has" method has returned False).
    """

    #: Fired when a change occurs that requires updating values.
    updated = Event

    def has_editor_value(self, model, row, column):
        """ Return whether or not the value can be edited.

        The default implementation is that cells that can be set are
        editable.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_editor_value : bool
            Whether or not the value is editable.
        """
        return model.can_set_value(row, column)

    def get_editor_value(self, model, row, column):
        """ Return a value suitable for editing.

        The default implementation is to return the underlying data value
        directly from the data model.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        value : any
            The value to edit.
        """
        return model.get_value(row, column)

    def set_editor_value(self, model, row, column, value):
        """ Set a value that is returned from editing.

        The default implementation is to set the value directly from the
        data model.  Returns True if successful, False if it fails.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.
        value : any
            The value to set.

        Raises
        -------
        DataViewSetError
            If the value cannot be set.
        """
        model.set_value(row, column, value)

    def has_text(self, model, row, column):
        """ Whether or not the value has a textual representation.

        The default implementation returns True if ``get_text``
        returns a non-empty value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_text : bool
            Whether or not the value has a textual representation.
        """
        return self.get_text(model, row, column) != ""

    def get_text(self, model, row, column):
        """ The textual representation of the underlying value.

        The default implementation calls str() on the underlying value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        text : str
            The textual representation of the underlying value.
        """
        return str(model.get_value(row, column))

    def set_text(self, model, row, column, text):
        """ Set the text of the underlying value.

        This is provided primarily for backends which may not permit
        non-text editing of values, in which case this provides an
        alternative route to setting the value.  The default implementation
        does not allow setting the text.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.
        text : str
            The text to set.

        Raises
        -------
        DataViewSetError
            If the value cannot be set.
        """
        raise DataViewSetError("Cannot set value.")

    def has_color(self, model, row, column):
        """ Whether or not the value has color data.

        has_color : bool
            Whether or not the value has data-associated color
            values.
        """
        return False

    def get_color(self, model, row, column):
        """ Get data-associated colour values for the given item.

        The default implementation returns white.

        color : Color instance
            The color associated with the cell.
        """
        return Color(rgba=(1.0, 1.0, 1.0, 1.0))

    def has_image(self, model, row, column):
        """ Whether or not the value has an image associated with it.

        The default implementation returns True if ``get_image``
        returns a non-None value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_image : bool
            Whether or not the value has an image associated with it.
        """
        return False

    def get_image(self, model, row, column):
        """ An image associated with the underlying value.

        The default implementation returns None.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        image : IImageResource
            The image associated with the underlying value.
        """
        from pyface.image_resource import ImageResource
        return ImageResource("image_not_found")

    def has_check_state(self, model, row, column):
        """ Whether or not the value has checked state.

        The default implementation returns False.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_check_state : bool
            Whether or not the value has a checked state.
        """
        return False

    def get_check_state(self, model, row, column):
        """ The state of the item check box.

        The default implementation returns "checked" if the value is
        truthy, or "unchecked" if the value is falsey.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        check_state : CheckState
            The current checked state.
        """
        return (
            CheckState.CHECKED
            if model.get_value(row, column)
            else CheckState.UNCHECKED
        )

    def set_check_state(self, model, row, column, check_state):
        """ Set the checked state of the underlying value.

        The default implementation does not allow setting the checked state.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.
        check_state : CheckState
            The check state value to set.

        Raises
        -------
        DataViewSetError
            If the value cannot be set.
        """
        raise DataViewSetError("Cannot set check state.")

    def has_tooltip(self, model, row, column):
        """ Whether or not the value has a tooltip.

        The default implementation returns True if ``get_tooltip``
        returns a non-empty value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_tooltip : bool
            Whether or not the value has a textual representation.
        """
        return self.get_tooltip(model, row, column) != ""

    def get_tooltip(self, model, row, column):
        """ The tooltip for the underlying value.

        The default implementation returns an empty string.

        tooltip : str
            The textual representation of the underlying value.
        """
        return ""

    @observe('+update_value_type')
    def update_value_type(self, event=None):
        """ Fire update event when marked traits change. """
        self.updated = True



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" The action controller for workbench menu and tool bars. """


from pyface.action.api import ActionController
from pyface.workbench.api import WorkbenchWindow
from traits.api import HasTraits, Instance


class ActionController(ActionController):
    """ The action controller for workbench menu and tool bars.

    The controller is used to 'hook' the invocation of every action on the menu
    and tool bars. This is done so that additional (and workbench specific)
    information can be added to action events. Currently, we attach a reference
    to the workbench window.

    """

    # 'ActionController' interface -----------------------------------------

    # The workbench window that this is the controller for.
    window = Instance(WorkbenchWindow)

    # ------------------------------------------------------------------------
    # 'ActionController' interface.
    # ------------------------------------------------------------------------

    def perform(self, action, event):
        """ Control an action invocation. """

        # Add a reference to the window and the application to the event.
        event.window = self.window

        return action.perform(event)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" Color conversion routines for the Qt toolkit.

This module provides a couple of utility methods to support the
pyface.color.Color class to_toolkit and from_toolkit methods.
"""

from pyface.qt.QtGui import QColor

from pyface.color import channels_to_ints, ints_to_channels


def toolkit_color_to_rgba(qcolor):
    """ Convert a QColor to an RGBA tuple.

    Parameters
    ----------
    qcolor : QColor
        A QColor object.

    Returns
    -------
    rgba_tuple : tuple
        A tuple of 4 floating point values between 0.0 and 1.0 inclusive.
    """
    values = (
        qcolor.red(),
        qcolor.green(),
        qcolor.blue(),
        qcolor.alpha(),
    )
    return ints_to_channels(values)


def rgba_to_toolkit_color(rgba):
    """ Convert an RGBA tuple to a QColor.

    Parameters
    ----------
    rgba_tuple : tuple
        A tuple of 4 floating point values between 0.0 and 1.0 inclusive.

    Returns
    -------
    qcolor : QColor
        A QColor object.
    """
    values = channels_to_ints(rgba)
    return QColor(*values)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from pyface.color import Color

from pyface.data_view.abstract_data_model import DataViewSetError
from .editable_value import EditableValue


class ColorValue(EditableValue):
    """ Editable value that presents a color value.

    This is suitable for use where the value returned by an item in
    the model is a Color object.
    """

    def is_valid(self, model, row, column, value):
        """ Is the given value valid for this item.

        The default implementation says the value must be a Color.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.
        value : any
            The value to test.

        Returns
        -------
        success : bool
            Whether or not the value is valid.
        """
        return isinstance(value, Color)

    def get_editor_value(self, model, row, column):
        """ Get the editable representation of the underlying value.

        The default uses a text hex representation of the color.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        text : str
            A hex string representation of the colour.
        """
        return model.get_value(row, column).hex()

    def set_editor_value(self, model, row, column, value):
        """ Set the editable representation of the underlying value.

        The default expects a string that can be parsed to a color value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.
        value : str
            A string that can be parsed to a color value.

        Returns
        -------
        success : bool
            Whether or not the value was successfully set.
        """
        try:
            color = Color.from_str(value)
        except Exception:
            raise DataViewSetError()
        return super().set_editor_value(model, row, column, color)

    def get_text(self, model, row, column):
        """ Get the textual representation of the underlying value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        text : str
            A hex string representation of the colour.
        """
        return model.get_value(row, column).hex()

    def set_text(self, model, row, column, text):
        """ Set the textual representation of the underlying value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.
        text : str
            The text to set.

        Returns
        -------
        success : bool
            Whether or not the value was successfully set.
        """
        return self.set_editor_value(model, row, column, text)

    def has_color(self, model, row, column):
        """ Whether or not the value has color data.

        The default implementation returns False.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_color : bool
            Whether or not the value has data-associated color
            values.
        """
        return True

    def get_color(self, model, row, column):
        """ Get data-associated colour values for the given item.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        color : Color instance
            The color associated with the cell.
        """
        return model.get_value(row, column)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from unittest import TestCase

from pyface.data_view.i_data_wrapper import text_format
from pyface.data_view.data_wrapper import DataWrapper


class TestDataWrapper(TestCase):

    def test_instantiate(self):
        data_wrapper = DataWrapper()
        self.assertEqual(data_wrapper.mimetypes(), set())

    def test_mimedata_roundtrip(self):
        data_wrapper = DataWrapper()
        data_wrapper.set_mimedata('text/plain', b'hello world')
        result = data_wrapper.get_mimedata('text/plain')

        self.assertEqual(data_wrapper.mimetypes(), {'text/plain'})
        self.assertEqual(result, b'hello world')

    def test_mimedata_overwrite(self):
        data_wrapper = DataWrapper()
        data_wrapper.set_mimedata('text/plain', b'hello world')
        data_wrapper.set_mimedata('text/plain', b'hello mars')
        result = data_wrapper.get_mimedata('text/plain')

        self.assertEqual(data_wrapper.mimetypes(), {'text/plain'})
        self.assertEqual(result, b'hello mars')

    def test_set_format(self):
        data_wrapper = DataWrapper()
        format = text_format()

        data_wrapper.set_format(format, 'hëllø wørld')
        result = data_wrapper.get_mimedata('text/plain')

        self.assertEqual(data_wrapper.mimetypes(), {'text/plain'})
        self.assertEqual(result, 'hëllø wørld'.encode('utf-8'))



# (C) Copyright 2007 Riverbank Computing Limited
# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


from pyface.qt import QtCore, QtGui


from traits.api import Any, Bool, HasTraits, Instance, provides


from pyface.i_widget import IWidget, MWidget


@provides(IWidget)
class Widget(MWidget, HasTraits):
    """ The toolkit specific implementation of a Widget.  See the IWidget
    interface for the API documentation.
    """

    # 'IWidget' interface ----------------------------------------------------

    #: The toolkit specific control that represents the widget.
    control = Any()

    #: The control's optional parent control.
    parent = Any()

    #: Whether or not the control is visible
    visible = Bool(True)

    #: Whether or not the control is enabled
    enabled = Bool(True)

    # Private interface ----------------------------------------------------

    #: The event filter for the widget.
    _event_filter = Instance(QtCore.QObject)

    # ------------------------------------------------------------------------
    # 'IWidget' interface.
    # ------------------------------------------------------------------------

    def show(self, visible):
        """ Show or hide the widget.

        Parameter
        ---------
        visible : bool
            Visible should be ``True`` if the widget should be shown.
        """
        self.visible = visible
        if self.control is not None:
            self.control.setVisible(visible)

    def enable(self, enabled):
        """ Enable or disable the widget.

        Parameter
        ---------
        enabled : bool
            The enabled state to set the widget to.
        """
        self.enabled = enabled
        if self.control is not None:
            self.control.setEnabled(enabled)

    def focus(self):
        """ Set the keyboard focus to this widget.
        """
        if self.control is not None:
            self.control.setFocus()

    def has_focus(self):
        """ Does the widget currently have keyboard focus?

        Returns
        -------
        focus_state : bool
            Whether or not the widget has keyboard focus.
        """
        return (
            self.control is not None
            and self.control.hasFocus()
        )

    def destroy(self):
        self._remove_event_listeners()
        if self.control is not None:
            self.control.hide()
            self.control.deleteLater()
            self.control = None

    def _add_event_listeners(self):
        self.control.installEventFilter(self._event_filter)

    def _remove_event_listeners(self):
        if self._event_filter is not None:
            if self.control is not None:
                self.control.removeEventFilter(self._event_filter)
            self._event_filter = None

    # Trait change handlers --------------------------------------------------

    def _visible_changed(self, new):
        if self.control is not None:
            self.show(new)

    def _enabled_changed(self, new):
        if self.control is not None:
            self.enable(new)

    def __event_filter_default(self):
        return WidgetEventFilter(self)


class WidgetEventFilter(QtCore.QObject):
    """ An internal class that watches for certain events on behalf of the
    Widget instance.

    This filter watches for show and hide events to make sure that visible
    state of the widget is the opposite of Qt's isHidden() state.  This is
    needed in case other code hides the toolkit widget
    """

    def __init__(self, widget):
        """ Initialise the event filter. """
        QtCore.QObject.__init__(self)
        self._widget = widget

    def eventFilter(self, obj, event):
        """ Adds any event listeners required by the window. """
        widget = self._widget
        # Sanity check.
        if obj is not widget.control:
            return False

        event_type = event.type()

        if event_type in {QtCore.QEvent.Show, QtCore.QEvent.Hide}:
            widget.visible = not widget.control.isHidden()

        return False



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
""" Abstract base class for all window actions. """


from pyface.window import Window
from traits.api import Instance, Property


from pyface.action.listening_action import ListeningAction


class WindowAction(ListeningAction):
    """ Abstract base class for window actions. """

    # 'ListeningAction' interface --------------------------------------------

    object = Property(observe="window")

    # 'WindowAction' interface -----------------------------------------------

    #: The window that the action is associated with.
    window = Instance(Window)

    # ------------------------------------------------------------------------
    # Protected interface.
    # ------------------------------------------------------------------------

    def _get_object(self):
        return self.window

    def destroy(self):
        # Disconnect listeners to window and dependent properties.
        self.window = None
        super(WindowAction, self).destroy()


class CloseWindowAction(WindowAction):
    """ Close the specified window """

    name = "Close"
    accelerator = "Ctrl+W"
    method = "close"



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Str, Union, observe

from pyface.ui_traits import PyfaceColor
from pyface.data_view.abstract_value_type import AbstractValueType
from pyface.ui_traits import Image


class ConstantValue(AbstractValueType):
    """ A value type that does not depend on the underlying data model.

    This value type is not editable, but the other data channels it
    provides can be modified by changing the appropriate trait on the
    value type.
    """

    #: The text value to display.
    text = Str(update_value_type=True)

    #: The color value to display or None if no color.
    color = Union(None, PyfaceColor)

    #: The image value to display.
    image = Image(update_value_type=True)

    #: The tooltip value to display.
    tooltip = Str(update_value_type=True)

    def has_editor_value(self, model, row, column):
        return False

    def get_text(self, model, row, column):
        return self.text

    def has_color(self, model, row, column):
        """ Whether or not the value has color data.

        Returns true if the supplied color is not None.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_color : bool
            Whether or not the value has data-associated color
            values.
        """
        return self.color is not None

    def get_color(self, model, row, column):
        """ Get data-associated colour values for the given item.

        The default implementation returns white.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        color : Color instance
            The color associated with the cell.
        """
        return self.color

    def has_image(self, model, row, column):
        return self.image is not None

    def get_image(self, model, row, column):
        if self.image is not None:
            return self.image
        return super().get_image(model, row, column)

    def get_tooltip(self, model, row, column):
        return self.tooltip

    @observe('color.rgba')
    def _color_updated(self, event):
        self.updated = True



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import (
    Any,
    Bool,
    Event,
    HasTraits,
    Interface,
    Instance,
    Property,
    Str,
    Vetoable,
    VetoableEvent,
    cached_property,
)


class IEditor(Interface):
    """ The base interface for all panes (central and dock) in a Task.
    """

    #: The editor's user-visible name.
    name = Str()

    #: The tooltip to show for the editor's tab, if any.
    tooltip = Str()

    #: The toolkit-specific control that represents the editor.
    control = Any()

    #: The object that the editor is editing.
    obj = Any()

    #: Has the editor's object been modified but not saved?
    dirty = Bool()

    #: The editor area to which the editor belongs.
    editor_area = Instance("pyface.tasks.i_editor_area_pane.IEditorAreaPane")

    #: Is the editor active in the editor area?
    is_active = Bool()

    #: Does the editor currently have the focus?
    has_focus = Bool()

    #: Fired when the editor has been requested to close.
    closing = VetoableEvent()

    #: Fired when the editor has been closed.
    closed = Event()

    # ------------------------------------------------------------------------
    # 'IEditor' interface.
    # ------------------------------------------------------------------------

    def close(self):
        """ Close the editor.
        """

    def create(self, parent):
        """ Create and set the toolkit-specific control that represents the
            editor.
        """

    def destroy(self):
        """ Destroy the toolkit-specific control that represents the editor.
        """


class MEditor(HasTraits):
    """ Mixin containing common code for toolkit-specific implementations.
    """

    # 'IEditor' interface -------------------------------------------------#

    name = Str()
    tooltip = Str()
    control = Any()
    obj = Any()
    dirty = Bool(False)

    editor_area = Instance("pyface.tasks.i_editor_area_pane.IEditorAreaPane")
    is_active = Property(Bool, observe="editor_area.active_editor")
    has_focus = Bool(False)

    closing = VetoableEvent()
    closed = Event()

    # ------------------------------------------------------------------------
    # 'IEditor' interface.
    # ------------------------------------------------------------------------

    def close(self):
        """ Close the editor.
        """
        if self.control is not None:
            self.closing = event = Vetoable()
            if not event.veto:
                self.editor_area.remove_editor(self)
                self.closed = True

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    @cached_property
    def _get_is_active(self):
        if self.editor_area is not None:
            return self.editor_area.active_editor == self
        return False



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from ..heading_text import HeadingText
from ..split_application_window import SplitApplicationWindow
from ..toolkit import toolkit_object

GuiTestAssistant = toolkit_object("util.gui_test_assistant:GuiTestAssistant")
no_gui_test_assistant = GuiTestAssistant.__name__ == "Unimplemented"


@unittest.skipIf(no_gui_test_assistant, "No GuiTestAssistant")
class TestSplitApplicationWindow(unittest.TestCase, GuiTestAssistant):
    def setUp(self):
        GuiTestAssistant.setUp(self)
        self.window = SplitApplicationWindow()

    def tearDown(self):
        if self.window.control is not None:
            with self.delete_widget(self.window.control):
                self.window.destroy()
        del self.window
        GuiTestAssistant.tearDown(self)

    def test_destroy(self):
        # test that destroy works even when no control
        with self.event_loop():
            self.window.destroy()

    def test_open_close(self):
        # test that opening and closing works as expected
        with self.assertTraitChanges(self.window, "opening", count=1):
            with self.assertTraitChanges(self.window, "opened", count=1):
                with self.event_loop():
                    self.window.open()

        with self.assertTraitChanges(self.window, "closing", count=1):
            with self.assertTraitChanges(self.window, "closed", count=1):
                with self.event_loop():
                    self.window.close()

    def test_horizontal_split(self):
        # test that horizontal split works
        self.window.direction = "horizontal"
        with self.assertTraitChanges(self.window, "opening", count=1):
            with self.assertTraitChanges(self.window, "opened", count=1):
                with self.event_loop():
                    self.window.open()

        with self.assertTraitChanges(self.window, "closing", count=1):
            with self.assertTraitChanges(self.window, "closed", count=1):
                with self.event_loop():
                    self.window.close()

    def test_contents(self):
        # test that contents works
        self.window.lhs = HeadingText
        self.window.rhs = HeadingText
        with self.assertTraitChanges(self.window, "opening", count=1):
            with self.assertTraitChanges(self.window, "opened", count=1):
                with self.event_loop():
                    self.window.open()

        with self.assertTraitChanges(self.window, "closing", count=1):
            with self.assertTraitChanges(self.window, "closed", count=1):
                with self.event_loop():
                    self.window.close()

    def test_ratio(self):
        # test that ratio split works
        self.window.ratio = 0.25
        with self.assertTraitChanges(self.window, "opening", count=1):
            with self.assertTraitChanges(self.window, "opened", count=1):
                with self.event_loop():
                    self.window.open()

        with self.assertTraitChanges(self.window, "closing", count=1):
            with self.assertTraitChanges(self.window, "closed", count=1):
                with self.event_loop():
                    self.window.close()



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" A view containing a main walter canvas. """


from pyface.workbench.api import View, WorkbenchWindow
from traits.api import HasTraits, Instance, Str, observe
from traitsui.api import View as TraitsView


class DebugViewModel(HasTraits):
    """ The model for the debug view! """

    # 'Model' interface ----------------------------------------------------

    active_editor = Str()
    active_part = Str()
    active_view = Str()

    window = Instance(WorkbenchWindow)

    # ------------------------------------------------------------------------
    # 'Model' interface.
    # ------------------------------------------------------------------------

    @observe("window.active_editor,window.active_part,window.active_view")
    def refresh(self, event):
        """ Refresh the model. """

        self.active_editor = self._get_id(self.window.active_editor)
        self.active_part = self._get_id(self.window.active_part)
        self.active_view = self._get_id(self.window.active_view)

    def _window_changed(self):
        """ Window changed! """

        self.refresh()

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _get_id(self, obj):
        """ Return the Id of an object. """

        if obj is None:
            id = "None"

        else:
            id = obj.id

        return id


class DebugView(View):
    """ A view containing a main walter canvas. """

    # 'IWorkbenchPart' interface -------------------------------------------

    # The part's name (displayed to the user).
    name = "Debug"

    # 'DebugView' interface ------------------------------------------------

    # The model for the debug view!
    model = Instance(DebugViewModel)

    # ------------------------------------------------------------------------
    # 'IWorkbenchPart' interface.
    # ------------------------------------------------------------------------

    def create_control(self, parent):
        """ Creates the toolkit-specific control that represents the view.

        'parent' is the toolkit-specific control that is the view's parent.

        """

        self.model = DebugViewModel(window=self.window)

        ui = self.model.edit_traits(
            parent=parent,
            kind="subpanel",
            view=TraitsView("active_part", "active_editor", "active_view"),
        )

        return ui.control



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" The interface for an image resource. """

from collections.abc import Sequence

from pyface.resource_manager import resource_manager
from pyface.resource.resource_path import resource_module, resource_path
from traits.api import HasTraits, Interface, List, Str


class IImageResource(Interface):
    """ The interface for an image resource.

    An image resource describes the location of an image and provides a way
    to create a toolkit-specific image on demand.
    """

    # 'ImageResource' interface --------------------------------------------

    #: The absolute path to the image.
    absolute_path = Str()

    #: The name of the image.
    name = Str()

    #: A list of directories, classes or instances that will be used to search
    #: for the image (see the resource manager for more details).
    search_path = List()

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, name, search_path=None):
        """ Creates a new image resource. """

    # ------------------------------------------------------------------------
    # 'ImageResource' interface.
    # ------------------------------------------------------------------------

    def create_image(self, size=None):
        """ Creates a toolkit specific image for this resource.

        Parameters
        ----------
        size : (int, int) or None
            The desired size as a width, height tuple, or None if wanting
            default image size.

        Returns
        -------
        image : toolkit image
            The toolkit image corresponding to the resource and the specified
            size.
        """

    # FIXME v3: The need to distinguish between bitmaps and images is toolkit
    # specific so, strictly speaking, the conversion to a bitmap should be done
    # wherever the toolkit actually needs it.
    def create_bitmap(self, size=None):
        """ Creates a toolkit specific bitmap for this resource.

        Parameters
        ----------
        size : (int, int) or None
            The desired size as a width, height tuple, or None if wanting
            default image size.

        Returns
        -------
        image : toolkit image
            The toolkit image corresponding to the resource and the specified
            size as a bitmap.
        """

    def create_icon(self, size=None):
        """ Creates a toolkit-specific icon for this resource.

        Parameters
        ----------
        size : (int, int) or None
            The desired size as a width, height tuple, or None if wanting
            default image size.

        Returns
        -------
        image : toolkit image
            The toolkit image corresponding to the resource and the specified
            size as an icon.
        """

    @classmethod
    def image_size(cls, image):
        """ Get the size of a toolkit image

        Parameters
        ----------
        image : toolkit image
            A toolkit image to compute the size of.

        Returns
        -------
        size : tuple
            The (width, height) tuple giving the size of the image.
        """


class MImageResource(HasTraits):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IImageResource interface.

    Implements: __init__(), create_image()
    """

    # Private interface ----------------------------------------------------

    #: The image-not-found image.  Note that it is not a trait.
    _image_not_found = None

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, name, search_path=None):
        self.name = name

        if isinstance(search_path, str):
            _path = [search_path]
        elif isinstance(search_path, Sequence):
            _path = search_path
        elif search_path is not None:
            _path = [search_path]
        else:
            _path = [resource_path()]
        self.search_path = _path + [resource_module()]

    # ------------------------------------------------------------------------
    # 'ImageResource' interface.
    # ------------------------------------------------------------------------

    def create_image(self, size=None):
        """ Creates a toolkit specific image for this resource.

        Parameters
        ----------
        size : (int, int) or None
            The desired size as a width, height tuple, or None if wanting
            default image size.

        Returns
        -------
        image : toolkit image
            The toolkit image corresponding to the resource and the specified
            size.
        """
        ref = self._get_ref(size)
        if ref is not None:
            image = ref.load()

        else:
            image = self._get_image_not_found_image()

        return image

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _get_ref(self, size=None):
        """ Return the resource manager reference to the image.

        Parameters
        ----------
        size : (int, int) or None
            The desired size as a width, height tuple, or None if wanting
            default image size.

        Returns
        -------
        ref : ImageReference instance
            The reference to the requested image.
        """

        if self._ref is None:
            self._ref = resource_manager.locate_image(
                self.name, self.search_path, size
            )

        return self._ref

    def _get_image_not_found_image(self):
        """ Returns the 'image not found' image.

        Returns
        -------
        image : toolkit image
            The 'not found' toolkit image.
        """

        not_found = self._get_image_not_found()

        if self is not not_found:
            image = not_found.create_image()

        else:
            raise ValueError("cannot locate the file for 'image_not_found'")

        return image

    @classmethod
    def _get_image_not_found(cls):
        """ Returns the 'image not found' image resource.

        Returns
        -------
        not_found : ImageResource instance
            An image resource for the the 'not found' image.
        """

        if cls._image_not_found is None:
            from pyface.image_resource import ImageResource

            cls._image_not_found = ImageResource("image_not_found")

        return cls._image_not_found



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The interface for a page in a wizard. """


from traits.api import Bool, HasTraits, Interface, Str, Tuple, Str


class IWizardPage(Interface):
    """ The interface for a page in a wizard. """

    # 'IWizardPage' interface ---------------------------------------------#

    # The unique Id of the page within the wizard.
    id = Str()

    # The Id of the next page.
    next_id = Str()

    # Set if this is the last page of the wizard.  It can be ignored for
    # simple linear wizards.
    last_page = Bool(False)

    # Is the page complete (i.e. should the 'Next' button be enabled)?
    complete = Bool(False)

    # The page heading.
    heading = Str()

    # The page sub-heading.
    subheading = Str()

    # The size of the page.
    size = Tuple()

    # ------------------------------------------------------------------------
    # 'IWizardPage' interface.
    # ------------------------------------------------------------------------

    def create_page(self, parent):
        """ Creates the wizard page. """

    def dispose_page(self):
        """ Disposes the wizard page.

        Subclasses are expected to override this method if they need to
        dispose of the contents of a page.
        """

    # ------------------------------------------------------------------------
    # Protected 'IWizardPage' interface.
    # ------------------------------------------------------------------------

    def _create_page_content(self, parent):
        """ Creates the actual page content. """


class MWizardPage(HasTraits):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IWizardPage interface.

    Implements: dispose_page()
    """

    # ------------------------------------------------------------------------
    # 'IWizardPage' interface.
    # ------------------------------------------------------------------------

    def dispose_page(self):
        """ Disposes the wizard page.

        Subclasses are expected to override this method if they need to
        dispose of the contents of a page.
        """

        pass



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The interface for all pyface wizards. """


from traits.api import Bool, HasTraits, Instance, List, Str
from pyface.i_dialog import IDialog


from .i_wizard_controller import IWizardController
from .i_wizard_page import IWizardPage


class IWizard(IDialog):
    """ The interface for all pyface wizards. """

    # 'IWizard' interface -------------------------------------------------#

    # The pages in the wizard.
    pages = List(IWizardPage)

    # The wizard controller provides the pages displayed in the wizard, and
    # determines when the wizard is complete etc.
    controller = Instance(IWizardController)

    # Should the 'Cancel' button be displayed?
    show_cancel = Bool(True)

    # 'IWindow' interface -------------------------------------------------#

    # The dialog title.
    title = Str("Wizard")

    # ------------------------------------------------------------------------
    # 'IWizard' interface.
    # ------------------------------------------------------------------------

    def next(self):
        """ Advance to the next page in the wizard. """

    def previous(self):
        """ Return to the previous page in the wizard. """


class MWizard(HasTraits):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IWizard interface.

    Implements: next(), previous()
    Reimplements: _create_contents()

    """

    # ------------------------------------------------------------------------
    # 'IWizard' interface.
    # ------------------------------------------------------------------------

    def next(self):
        """ Advance to the next page in the wizard. """

        page = self.controller.get_next_page(self.controller.current_page)
        self._show_page(page)

    def previous(self):
        """ Return to the previous page in the wizard. """

        page = self.controller.get_previous_page(self.controller.current_page)
        self._show_page(page)

        return

    # ------------------------------------------------------------------------
    # Protected 'IWindow' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        """ Creates the window contents. """

        # This creates the dialog and button areas.
        super(MWizard, self)._create_contents(parent)

        # Wire up the controller.
        self._initialize_controller(self.controller)

        # Show the first page.
        self._show_page(self.controller.get_first_page())

        return

    # ------------------------------------------------------------------------
    # Protected MWizard interface.
    # ------------------------------------------------------------------------

    def _show_page(self, page):
        """ Show the specified page. """

        # Set the current page in the controller.
        #
        # fixme: Shouldn't this interface be reversed?  Maybe calling
        # 'next_page' on the controller should cause it to set its own current
        # page?
        self.controller.current_page = page

    def _update(self, event):
        """ Enables/disables buttons depending on the state of the wizard. """

        pass

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _initialize_controller(self, controller):
        """ Initializes the wizard controller. """

        controller.observe(self._update, "complete")

        controller.observe(self._on_current_page_changed, "current_page")

        return

    # Trait event handlers -------------------------------------------------

    def _on_current_page_changed(self, event):
        """ Called when the current page is changed. """

        if event.old is not None:
            event.old.observe(self._update, "complete", remove=True)

        if event.new is not None:
            event.new.observe(self._update, "complete")

        self._update(event=None)

    def _on_closed_changed(self):
        """ Called when the wizard is closed. """

        self.controller.dispose_pages()

        return



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import os
import platform
import unittest

from importlib_resources import files

import pyface
import pyface.tests
from ..image_resource import ImageResource
from ..toolkit import toolkit_object


is_qt = toolkit_object.toolkit == "qt4"
if is_qt:
    from pyface.qt import qt_api
is_pyqt4_windows = (
    is_qt and qt_api == "pyqt" and platform.system() == "Windows"
)


SEARCH_PATH = os.fspath(files("pyface") / "images")
IMAGE_PATH = os.fspath(files("pyface.tests") / "images" / "core.png")


class TestImageResource(unittest.TestCase):
    def setUp(self):
        # clear cached "not found" image
        ImageResource._image_not_found = None

    def test_create_image(self):
        image_resource = ImageResource("core")
        image = image_resource.create_image()
        self.assertIsNotNone(image)
        self.assertEqual(image_resource.absolute_path, IMAGE_PATH)

    def test_create_image_again(self):
        image_resource = ImageResource("core")
        image = image_resource.create_image()
        self.assertIsNotNone(image)
        self.assertEqual(image_resource.absolute_path, IMAGE_PATH)

    def test_create_image_search_path(self):
        image_resource = ImageResource("splash", [SEARCH_PATH])
        self.assertEqual(
            image_resource.search_path, [SEARCH_PATH, pyface.tests]
        )
        image = image_resource.create_image()
        self.assertIsNotNone(image)
        self.assertEqual(
            image_resource.absolute_path,
            os.path.join(SEARCH_PATH, "splash.png"),
        )

    def test_create_image_search_path_string(self):
        image_resource = ImageResource("splash", SEARCH_PATH)
        self.assertEqual(
            image_resource.search_path, [SEARCH_PATH, pyface.tests]
        )
        image = image_resource.create_image()
        self.assertIsNotNone(image)
        self.assertEqual(
            image_resource.absolute_path,
            os.path.join(SEARCH_PATH, "splash.png"),
        )

    def test_create_image_missing(self):
        image_resource = ImageResource("doesnt_exist.png")
        image = image_resource.create_image()
        self.assertIsNotNone(image)
        self.assertIsNotNone(image_resource._image_not_found)

    def test_create_bitmap(self):
        image_resource = ImageResource("core.png")
        image = image_resource.create_bitmap()
        self.assertIsNotNone(image)
        self.assertEqual(image_resource.absolute_path, IMAGE_PATH)

    def test_create_icon(self):
        image_resource = ImageResource("core.png")
        image = image_resource.create_icon()
        self.assertIsNotNone(image)
        self.assertEqual(image_resource.absolute_path, IMAGE_PATH)

    def test_image_size(self):
        image_resource = ImageResource("core")
        image = image_resource.create_image()
        size = image_resource.image_size(image)
        self.assertEqual(image_resource._ref.filename, IMAGE_PATH)
        self.assertEqual(size, (64, 64))

    @unittest.skipIf(
        is_pyqt4_windows, "QPixmap bug returns (0, 0).  Issue #301."
    )  # noqa
    def test_image_size_search_path(self):
        image_resource = ImageResource("splash", [SEARCH_PATH])
        image = image_resource.create_image()
        size = image_resource.image_size(image)
        self.assertEqual(
            image_resource.absolute_path,
            os.path.join(SEARCH_PATH, "splash.png"),
        )
        self.assertEqual(size, (601, 203))



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Either, List, Str, Tuple, Enum, Instance


from pyface.tasks.task_layout import LayoutContainer, TaskLayout


class TaskWindowLayout(LayoutContainer):
    """ The layout of a TaskWindow.
    """

    #: The ID of the active task. If unspecified, the first task will be
    #: active.
    active_task = Str()

    #: The tasks contained in the window. If an ID is specified, the task will
    #: use its default layout. Otherwise, it will use the specified TaskLayout
    items = List(Either(Str, Instance(TaskLayout)), pretty_skip=True)

    #: The position of the window.
    position = Tuple(-1, -1)

    #: The size of the window.
    size = Tuple(800, 600)

    #: Whether or not the application is maximized.
    size_state = Enum("normal", "maximized")

    def get_active_task(self):
        """ Returns the ID of the active task in the layout, or None if there is
            no active task.
        """
        if self.active_task:
            return self.active_task
        elif self.items:
            first = self.items[0]
            return first if isinstance(first, str) else first.id
        return None

    def get_tasks(self):
        """ Returns the IDs of the tasks in the layout.
        """
        return [
            (item if isinstance(item, str) else item.id) for item in self.items
        ]

    def is_equivalent_to(self, layout):
        """ Returns whether two layouts are equivalent, i.e. whether they
            contain the same tasks.
        """
        return isinstance(layout, TaskWindowLayout) and set(
            self.get_tasks()
        ) == set(layout.get_tasks())



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A window that is split in two either horizontally or vertically. """


from pyface.application_window import ApplicationWindow
from pyface.split_widget import SplitWidget


class SplitApplicationWindow(ApplicationWindow, SplitWidget):
    """ A window that is split in two either horizontally or vertically. """

    # ------------------------------------------------------------------------
    # Protected 'ApplicationWindow' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        """ Creates the window contents.

        Parameters
        ----------
        parent : toolkit control
            The window's toolkit control to be used as the parent for the
            splitter control.

        Returns
        -------
        control : toolkit control
            The splitter control to be used for contents of the window.
        """
        return self._create_splitter(parent)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from pyface.toolkit import toolkit
from ..text_field import TextField
from .field_mixin import FieldMixin


is_wx = toolkit.toolkit == "wx"


class TestTextField(FieldMixin, unittest.TestCase):
    def _create_widget(self):
        return TextField(
            parent=self.parent.control, value="test", tooltip="Dummy"
        )

    # Tests ------------------------------------------------------------------

    def test_text_field(self):
        self._create_widget_control()

        self.widget.value = "new value"
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), "new value")

    def test_text_field_set(self):
        self._create_widget_control()

        with self.assertTraitChanges(self.widget, "value", count=1):
            self.widget._set_control_value("new value")
            self.gui.process_events()

        self.assertEqual(self.widget.value, "new value")

    def test_text_field_echo(self):
        self.widget.echo = "password"
        self._create_widget_control()

        self.assertEqual(self.widget._get_control_echo(), "password")

    @unittest.skipIf(
        is_wx, "Can't change password mode for wx after control " "creation."
    )
    def test_text_field_echo_change(self):
        self._create_widget_control()

        self.widget.echo = "password"
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_echo(), "password")

    def test_text_field_placeholder(self):
        self._create_widget_control()

        self.widget.placeholder = "test"
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_placeholder(), "test")

    def test_text_field_readonly(self):
        self.widget.read_only = True
        self._create_widget_control()

        self.gui.process_events()

        self.assertEqual(self.widget._get_control_read_only(), True)

    @unittest.skipIf(
        is_wx, "Can't change read_only mode for wx after control " "creation."
    )
    def test_text_field_readonly_change(self):
        self._create_widget_control()

        self.widget.read_only = True
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_read_only(), True)
